@@ -4,7 +4,10 @@ use crate::{
     ast::{Publicity, SrcSpan, UnqualifiedImport, UntypedImport},
     build::Origin,
     reference::{EntityKind, ReferenceKind},
-    type_::{Environment, Error, ModuleInterface, Problems, ValueConstructorVariant},
+    type_::{
+        Environment, Error, ModuleInterface, Problems, ValueConstructorVariant,
+        error::InvalidImportKind,
+    },
 };
 
 use super::Imported;
@@ -56,7 +59,7 @@ impl<'context, 'problems> Importer<'context, 'problems> {
             return;
         };
 
-        if let Err(e) = self.check_src_does_not_import_test(module_info, location, name.clone()) {
+        if let Err(e) = self.check_for_invalid_imports(module_info, location, name.clone()) {
             self.problems.error(e);
             return;
         }
@@ -67,15 +70,21 @@ impl<'context, 'problems> Importer<'context, 'problems> {
         }
 
         // Insert unqualified imports into scope
+        let module_name = &module_info.name;
         for type_ in &import.unqualified_types {
-            self.register_unqualified_type(type_, module_info);
+            self.register_unqualified_type(type_, module_name.clone(), module_info);
         }
         for value in &import.unqualified_values {
-            self.register_unqualified_value(value, module_info);
+            self.register_unqualified_value(value, module_name.clone(), module_info);
         }
     }
 
-    fn register_unqualified_type(&mut self, import: &UnqualifiedImport, module: &ModuleInterface) {
+    fn register_unqualified_type(
+        &mut self,
+        import: &UnqualifiedImport,
+        module_name: EcoString,
+        module: &ModuleInterface,
+    ) {
         let imported_name = import.as_name.as_ref().unwrap_or(&import.name);
 
         // Register the unqualified import if it is a type constructor
@@ -101,7 +110,9 @@ impl<'context, 'problems> Importer<'context, 'problems> {
 
         self.environment.references.register_type(
             imported_name.clone(),
-            EntityKind::ImportedType,
+            EntityKind::ImportedType {
+                module: module_name,
+            },
             import.location,
             Publicity::Private,
         );
@@ -122,7 +133,12 @@ impl<'context, 'problems> Importer<'context, 'problems> {
         }
     }
 
-    fn register_unqualified_value(&mut self, import: &UnqualifiedImport, module: &ModuleInterface) {
+    fn register_unqualified_value(
+        &mut self,
+        import: &UnqualifiedImport,
+        module_name: EcoString,
+        module: &ModuleInterface,
+    ) {
         let import_name = &import.name;
         let location = import.location;
         let used_name = import.as_name.as_ref().unwrap_or(&import.name);
@@ -172,7 +188,9 @@ impl<'context, 'problems> Importer<'context, 'problems> {
                 );
                 self.environment.references.register_value(
                     used_name.clone(),
-                    EntityKind::ImportedConstructor,
+                    EntityKind::ImportedConstructor {
+                        module: module_name,
+                    },
                     location,
                     Publicity::Private,
                 );
@@ -189,7 +207,9 @@ impl<'context, 'problems> Importer<'context, 'problems> {
             | ValueConstructorVariant::ModuleFn { module, .. } => {
                 self.environment.references.register_value(
                     used_name.clone(),
-                    EntityKind::ImportedValue,
+                    EntityKind::ImportedValue {
+                        module: module_name,
+                    },
                     location,
                     Publicity::Private,
                 );
@@ -222,20 +242,28 @@ impl<'context, 'problems> Importer<'context, 'problems> {
             .insert(used_name.clone(), location);
     }
 
-    fn check_src_does_not_import_test(
+    /// Check for invalid imports, such as `src` importing `test` or `dev`.
+    fn check_for_invalid_imports(
         &mut self,
         module_info: &ModuleInterface,
         location: SrcSpan,
         imported_module: EcoString,
     ) -> Result<(), Error> {
-        if self.origin.is_src() && !module_info.origin.is_src() {
-            return Err(Error::SrcImportingTest {
-                location,
-                src_module: self.environment.current_module.clone(),
-                test_module: imported_module,
-            });
-        }
-        Ok(())
+        let kind = match (self.origin, module_info.origin) {
+            // `src` cannot import `test` or `dev`
+            (Origin::Src, Origin::Test) => InvalidImportKind::SrcImportingTest,
+            (Origin::Src, Origin::Dev) => InvalidImportKind::SrcImportingDev,
+            // `dev` cannot import `test`
+            (Origin::Dev, Origin::Test) => InvalidImportKind::DevImportingTest,
+            _ => return Ok(()),
+        };
+
+        Err(Error::InvalidImport {
+            location,
+            importing_module: self.environment.current_module.clone(),
+            imported_module,
+            kind,
+        })
     }
 
     fn register_module(
@@ -246,20 +274,18 @@ impl<'context, 'problems> Importer<'context, 'problems> {
         if let Some(used_name) = import.used_name() {
             self.check_not_a_duplicate_import(&used_name, import.location)?;
 
-            if import.unqualified_types.is_empty() && import.unqualified_values.is_empty() {
-                // When the module has no unqualified imports, we track its usage
-                // so we can warn if not used by the end of the type checking
-
-                self.environment.references.register_module(
+            if let Some(alias_location) = import.alias_location() {
+                self.environment.references.register_aliased_module(
                     used_name.clone(),
-                    EntityKind::ImportedModule,
+                    import.module.clone(),
+                    alias_location,
                     import.location,
                 );
-            } else if let Some(alias_location) = import.alias_location() {
+            } else {
                 self.environment.references.register_module(
                     used_name.clone(),
-                    EntityKind::ModuleAlias,
-                    alias_location,
+                    import.module.clone(),
+                    import.location,
                 );
             }
 
