@@ -3,7 +3,11 @@ use std::sync::OnceLock;
 use type_::{FieldMap, TypedCallArg};
 
 use super::*;
-use crate::type_::{HasType, Type, ValueConstructorVariant, bool};
+use crate::{
+    build::ExpressionPosition,
+    exhaustiveness::CompiledCase,
+    type_::{HasType, Type, ValueConstructorVariant, bool},
+};
 
 #[derive(Debug, Clone, PartialEq, Eq)]
 pub enum TypedExpr {
@@ -57,6 +61,7 @@ pub enum TypedExpr {
         args: Vec<TypedArg>,
         body: Vec1<TypedStatement>,
         return_annotation: Option<TypeAst>,
+        purity: Purity,
     },
 
     List {
@@ -87,6 +92,7 @@ pub enum TypedExpr {
         type_: Arc<Type>,
         subjects: Vec<Self>,
         clauses: Vec<Clause<Self, Arc<Type>, EcoString>>,
+        compiled_case: CompiledCase,
     },
 
     RecordAccess {
@@ -158,7 +164,7 @@ pub enum TypedExpr {
     RecordUpdate {
         location: SrcSpan,
         type_: Arc<Type>,
-        record: TypedAssignment,
+        record: Box<TypedAssignment>,
         constructor: Box<Self>,
         args: Vec<CallArg<Self>>,
     },
@@ -334,7 +340,7 @@ impl TypedExpr {
 
             Self::Call { fun, args, .. } => args
                 .iter()
-                .find_map(|arg| arg.find_node(byte_index))
+                .find_map(|arg| arg.find_node(byte_index, fun, args))
                 .or_else(|| fun.find_node(byte_index))
                 .or_else(|| self.self_if_contains_location(byte_index)),
 
@@ -364,10 +370,15 @@ impl TypedExpr {
                 .find_map(|arg| arg.find_node(byte_index))
                 .or_else(|| self.self_if_contains_location(byte_index)),
 
-            Self::RecordUpdate { record, args, .. } => args
+            Self::RecordUpdate {
+                record,
+                constructor,
+                args,
+                ..
+            } => args
                 .iter()
                 .filter(|arg| arg.implicit.is_none())
-                .find_map(|arg| arg.find_node(byte_index))
+                .find_map(|arg| arg.find_node(byte_index, constructor, args))
                 .or_else(|| record.find_node(byte_index))
                 .or_else(|| self.self_if_contains_location(byte_index)),
         }
@@ -653,16 +664,48 @@ impl TypedExpr {
 
     pub fn is_literal(&self) -> bool {
         match self {
-            Self::Int { .. }
-            | Self::List { .. }
-            | Self::Float { .. }
-            | Self::Tuple { .. }
-            | Self::String { .. }
-            | Self::BitArray { .. } => true,
+            Self::Int { .. } | Self::Float { .. } | Self::String { .. } => true,
+
+            Self::List { elements, .. } | Self::Tuple { elements, .. } => {
+                elements.iter().all(|value| value.is_literal())
+            }
+
+            Self::BitArray { segments, .. } => {
+                segments.iter().all(|segment| segment.value.is_literal())
+            }
+
+            // Calls are literals if they are records and all the arguemnts are also literals.
+            Self::Call { fun, args, .. } => {
+                fun.is_record_constructor()
+                    && args.iter().all(|argument| argument.value.is_literal())
+            }
+
+            // Variables are literals if they are record constructors that take no arguments.
+            Self::Var {
+                constructor:
+                    ValueConstructor {
+                        variant: ValueConstructorVariant::Record { arity: 0, .. },
+                        ..
+                    },
+                ..
+            } => true,
+
             _ => false,
         }
     }
 
+    pub fn is_known_value(&self) -> bool {
+        match self {
+            TypedExpr::BinOp { left, right, .. } => left.is_known_value() && right.is_known_value(),
+
+            TypedExpr::NegateBool { value, .. } | TypedExpr::NegateInt { value, .. } => {
+                value.is_known_value()
+            }
+
+            _ => self.is_literal(),
+        }
+    }
+
     pub fn is_literal_string(&self) -> bool {
         match self {
             Self::String { .. } => true,
@@ -757,23 +800,28 @@ impl TypedExpr {
             TypedExpr::ModuleSelect { .. } => true,
 
             // A pipeline is a pure value constructor if its last step is a record builder,
-            // or a call to a fn expression that has a body comprised of just pure value
-            // constructors. For example:
+            // or a call to a pure function. For example:
             //  - `wibble() |> wobble() |> Ok`
             //  - `"hello" |> fn(s) { s <> " world!" }`
-            TypedExpr::Pipeline { finally, .. } => match finally.as_ref() {
-                TypedExpr::Fn { body, .. } => body.iter().all(|s| s.is_pure_value_constructor()),
-                fun => fun.is_pure_value_constructor(),
-            },
+            TypedExpr::Pipeline {
+                first_value,
+                assignments,
+                finally,
+                ..
+            } => {
+                first_value.value.is_pure_value_constructor()
+                    && assignments
+                        .iter()
+                        .all(|(assignment, _)| assignment.value.is_pure_value_constructor())
+                    && finally.is_pure_value_constructor()
+            }
 
-            TypedExpr::Call { fun, .. } => match fun.as_ref() {
-                // Immediately calling a fn expression that has a body comprised of just
-                // pure value constructors is in itself pure.
-                TypedExpr::Fn { body, .. } => body.iter().all(|s| s.is_pure_value_constructor()),
-                // And calling a record builder is a pure value constructor:
-                // `Some(1)`
-                fun => fun.is_record_builder(),
-            },
+            TypedExpr::Call { fun, args, .. } => {
+                (fun.is_record_builder() || fun.called_function_purity().is_pure())
+                    && args
+                        .iter()
+                        .all(|argument| argument.value.is_pure_value_constructor())
+            }
 
             // A block is pure if all the statements it's made of are pure.
             // For example `{ True 1 }`
@@ -806,6 +854,81 @@ impl TypedExpr {
         }
     }
 
+    /// Returns the purity of the left hand side of a function call. For example:
+    ///
+    /// ```gleam
+    /// io.println("Hello, world!")
+    /// ```
+    ///
+    /// Here, the left hand side is `io.println`, which is an impure function,
+    /// so we would return `Purity::Impure`.
+    ///
+    /// This does not check whether an expression is pure on its own; for that
+    /// see `is_pure_value_constructor`.
+    ///
+    pub fn called_function_purity(&self) -> Purity {
+        match self {
+            TypedExpr::Var { constructor, .. } => constructor.called_function_purity(),
+            TypedExpr::ModuleSelect { constructor, .. } => constructor.called_function_purity(),
+            TypedExpr::Fn { purity, .. } => *purity,
+
+            // While we can infer the purity of some of these expressions, such
+            // as `Case`, in this example:
+            //  ```gleam
+            // case x {
+            //   True -> io.println
+            //   False -> function.identity
+            // }("Hello")
+            // ```
+            //
+            // This kind of code is rare in real Gleam applications, and as this
+            // system is just used for warnings, it is unlikely that supporting
+            // them will provide any significant benefit to developer experience,
+            // so we just return `Unknown` for simplicity.
+            //
+            TypedExpr::Block { .. }
+            | TypedExpr::Pipeline { .. }
+            | TypedExpr::Call { .. }
+            | TypedExpr::Case { .. }
+            | TypedExpr::RecordAccess { .. }
+            | TypedExpr::TupleIndex { .. }
+            | TypedExpr::Echo { .. } => Purity::Unknown,
+
+            // The following expressions are all invalid on the left hand side
+            // of a call expression: `10()` is not valid Gleam. Therefore, we
+            // don't really care about any of these as they shouldn't appear in
+            // well typed Gleam code, and so we can just return `Unknown`.
+            TypedExpr::Int { .. }
+            | TypedExpr::Float { .. }
+            | TypedExpr::String { .. }
+            | TypedExpr::List { .. }
+            | TypedExpr::BinOp { .. }
+            | TypedExpr::Tuple { .. }
+            | TypedExpr::Todo { .. }
+            | TypedExpr::Panic { .. }
+            | TypedExpr::BitArray { .. }
+            | TypedExpr::RecordUpdate { .. }
+            | TypedExpr::NegateBool { .. }
+            | TypedExpr::NegateInt { .. }
+            | TypedExpr::Invalid { .. } => Purity::Unknown,
+        }
+    }
+
+    #[must_use]
+    pub fn is_record_constructor(&self) -> bool {
+        match self {
+            TypedExpr::Var {
+                constructor:
+                    ValueConstructor {
+                        variant: ValueConstructorVariant::Record { .. },
+                        ..
+                    },
+                ..
+            } => true,
+            _ => false,
+        }
+    }
+
     #[must_use]
     pub fn is_record_builder(&self) -> bool {
         match self {
@@ -819,6 +942,7 @@ impl TypedExpr {
         }
     }
 
+    #[must_use]
     /// If `self` is a record constructor, returns the nuber of arguments it
     /// needs to be called. Otherwise, returns `None`.
     ///
@@ -932,8 +1056,11 @@ impl TypedExpr {
 }
 
 impl<'a> From<&'a TypedExpr> for Located<'a> {
-    fn from(value: &'a TypedExpr) -> Self {
-        Located::Expression(value)
+    fn from(expression: &'a TypedExpr) -> Self {
+        Located::Expression {
+            expression,
+            position: ExpressionPosition::Expression,
+        }
     }
 }
 
@@ -949,13 +1076,12 @@ impl HasType for TypedExpr {
     }
 }
 
-impl crate::bit_array::GetLiteralValue for TypedExpr {
-    fn as_int_literal(&self) -> Option<i64> {
-        if let TypedExpr::Int { value: val, .. } = self {
-            if let Ok(val) = val.parse::<i64>() {
-                return Some(val);
-            }
+impl bit_array::GetLiteralValue for TypedExpr {
+    fn as_int_literal(&self) -> Option<BigInt> {
+        if let TypedExpr::Int { int_value, .. } = self {
+            Some(int_value.clone())
+        } else {
+            None
         }
-        None
     }
 }
