@@ -58,6 +58,7 @@ pub enum UntypedExpr {
     BinOp {
         location: SrcSpan,
         name: BinOp,
+        name_location: SrcSpan,
         left: Box<Self>,
         right: Box<Self>,
     },
@@ -88,7 +89,7 @@ pub enum UntypedExpr {
 
     Tuple {
         location: SrcSpan,
-        elems: Vec<Self>,
+        elements: Vec<Self>,
     },
 
     TupleIndex {
@@ -149,7 +150,10 @@ pub enum UntypedExpr {
 impl UntypedExpr {
     pub fn location(&self) -> SrcSpan {
         match self {
-            Self::PipeLine { expressions, .. } => expressions.last().location(),
+            Self::PipeLine { expressions, .. } => expressions
+                .first()
+                .location()
+                .merge(&expressions.last().location()),
 
             Self::Fn { location, .. }
             | Self::Var { location, .. }
