@@ -16,6 +16,7 @@ use ecow::EcoString;
 pub use environment::*;
 pub use error::{Error, Problems, UnifyErrorSituation, Warning};
 pub(crate) use expression::ExprTyper;
+use expression::Purity;
 pub use fields::FieldMap;
 use hexpm::version::Version;
 pub use prelude::*;
@@ -165,13 +166,6 @@ impl Type {
         }
     }
 
-    pub fn is_type_variable(&self) -> bool {
-        match self {
-            Self::Var { type_ } => type_.borrow().is_variable(),
-            _ => false,
-        }
-    }
-
     pub fn return_type(&self) -> Option<Arc<Self>> {
         match self {
             Self::Fn { return_, .. } => Some(return_.clone()),
@@ -269,6 +263,18 @@ impl Type {
         }
     }
 
+    pub fn is_utf_codepoint(&self) -> bool {
+        match self {
+            Self::Named { module, name, .. }
+                if "UtfCodepoint" == name && is_prelude_module(module) =>
+            {
+                true
+            }
+            Self::Var { type_ } => type_.borrow().is_nil(),
+            _ => false,
+        }
+    }
+
     pub fn is_bool(&self) -> bool {
         match self {
             Self::Named { module, name, .. } if "Bool" == name && is_prelude_module(module) => true,
@@ -564,6 +570,19 @@ impl Type {
             }
         }
     }
+
+    pub(crate) fn is_non_public_named_type(&self) -> bool {
+        match self {
+            Type::Named { publicity, .. } => match publicity {
+                Publicity::Public => false,
+                Publicity::Private => true,
+                Publicity::Internal { .. } => true,
+            },
+            Type::Fn { .. } => false,
+            Type::Var { .. } => false,
+            Type::Tuple { .. } => false,
+        }
+    }
 }
 
 impl TypeVar {
@@ -672,6 +691,7 @@ pub enum ValueConstructorVariant {
         implementations: Implementations,
         external_erlang: Option<(EcoString, EcoString)>,
         external_javascript: Option<(EcoString, EcoString)>,
+        purity: Purity,
     },
 
     /// A constructor for a custom type
@@ -737,6 +757,7 @@ impl ValueConstructorVariant {
                 documentation: None,
                 location: *location,
                 field_map: None,
+                purity: Purity::Impure,
             },
 
             Self::ModuleFn {
@@ -747,6 +768,7 @@ impl ValueConstructorVariant {
                 field_map,
                 external_erlang,
                 external_javascript,
+                purity,
                 ..
             } => ModuleValueConstructor::Fn {
                 name: name.clone(),
@@ -756,6 +778,7 @@ impl ValueConstructorVariant {
                 external_javascript: external_javascript.clone(),
                 location: *location,
                 field_map: field_map.clone(),
+                purity: *purity,
             },
         }
     }
@@ -871,6 +894,7 @@ pub enum ModuleValueConstructor {
         external_javascript: Option<(EcoString, EcoString)>,
         field_map: Option<FieldMap>,
         documentation: Option<EcoString>,
+        purity: Purity,
     },
 
     Constant {
@@ -896,6 +920,38 @@ impl ModuleValueConstructor {
             | ModuleValueConstructor::Constant { documentation, .. } => documentation.as_deref(),
         }
     }
+
+    /// Returns the purity of this value constructor if it is called as a function.
+    /// Referencing a module value by itself is always pure, but calling is as a
+    /// function might not be.
+    pub fn called_function_purity(&self) -> Purity {
+        match self {
+            // If we call a module constant or local variable as a function, we
+            // no longer have enough information to determine its purity. For
+            // example:
+            //
+            // ```gleam
+            // const function1 = io.println
+            // const function2 = function.identity
+            //
+            // pub fn main() {
+            //   function1("Hello")
+            //   function2("Hello")
+            // }
+            // ```
+            //
+            // At this point, we don't have any information about the purity of
+            // the `function1` and `function2` functions, and must return
+            // `Purity::Unknown`. See the documentation for the `Purity` type
+            // for more information on why this is the case.
+            ModuleValueConstructor::Constant { .. } => Purity::Unknown,
+
+            // Constructing records is always pure
+            ModuleValueConstructor::Record { .. } => Purity::Pure,
+
+            ModuleValueConstructor::Fn { purity, .. } => *purity,
+        }
+    }
 }
 
 #[derive(Debug, Clone)]
@@ -1037,17 +1093,15 @@ impl ModuleInterface {
     }
 
     pub fn get_main_function(&self, target: Target) -> Result<ModuleFunction, crate::Error> {
-        let not_found = || crate::Error::ModuleDoesNotHaveMainFunction {
-            module: self.name.clone(),
+        // Module must have a value with the name "main"
+        let Some(value) = self.values.get(&EcoString::from("main")) else {
+            return Err(crate::Error::ModuleDoesNotHaveMainFunction {
+                module: self.name.clone(),
+                origin: self.origin,
+            });
         };
 
-        // Module must have a value with the name "main"
-        let value = self
-            .values
-            .get(&EcoString::from("main"))
-            .ok_or_else(not_found)?;
-
-        assert_suitable_main_function(value, &self.name, target)?;
+        assert_suitable_main_function(value, &self.name, self.origin, target)?;
 
         Ok(ModuleFunction {
             package: self.package.clone(),
@@ -1134,7 +1188,7 @@ impl TypeVar {
     pub fn is_variable(&self) -> bool {
         match self {
             Self::Unbound { .. } | Self::Generic { .. } => true,
-            Self::Link { type_ } => type_.is_type_variable(),
+            Self::Link { type_ } => type_.is_variable(),
         }
     }
 
@@ -1276,6 +1330,7 @@ pub struct TypeConstructor {
     pub deprecation: Deprecation,
     pub documentation: Option<EcoString>,
 }
+
 impl TypeConstructor {
     pub(crate) fn with_location(mut self, location: SrcSpan) -> Self {
         self.origin = location;
@@ -1314,6 +1369,15 @@ impl Default for Deprecation {
 }
 
 impl ValueConstructor {
+    pub fn local_variable(location: SrcSpan, origin: VariableOrigin, type_: Arc<Type>) -> Self {
+        Self {
+            publicity: Publicity::Private,
+            deprecation: Deprecation::NotDeprecated,
+            variant: ValueConstructorVariant::LocalVariable { location, origin },
+            type_,
+        }
+    }
+
     pub fn is_local_variable(&self) -> bool {
         self.variant.is_local_variable()
     }
@@ -1357,6 +1421,40 @@ impl ValueConstructor {
             }
         }
     }
+
+    /// Returns the purity of this value constructor if it is called as a function.
+    /// Referencing a value constructor by itself is always pure, but calling is as a
+    /// function might not be.
+    pub fn called_function_purity(&self) -> Purity {
+        match &self.variant {
+            // If we call a module constant or local variable as a function, we
+            // no longer have enough information to determine its purity. For
+            // example:
+            //
+            // ```gleam
+            // const function1 = io.println
+            // const function2 = function.identity
+            //
+            // pub fn main() {
+            //   function1("Hello")
+            //   function2("Hello")
+            // }
+            // ```
+            //
+            // At this point, we don't have any information about the purity of
+            // the `function1` and `function2` functions, and must return
+            // `Purity::Unknown`. See the documentation for the `Purity` type
+            // for more information on why this is the case.
+            ValueConstructorVariant::LocalVariable { .. }
+            | ValueConstructorVariant::ModuleConstant { .. }
+            | ValueConstructorVariant::LocalConstant { .. } => Purity::Unknown,
+
+            // Constructing records is always pure
+            ValueConstructorVariant::Record { .. } => Purity::Pure,
+
+            ValueConstructorVariant::ModuleFn { purity, .. } => *purity,
+        }
+    }
 }
 
 #[derive(Debug, Clone, PartialEq, Eq)]
@@ -1553,12 +1651,9 @@ pub enum FieldAccessUsage {
 fn assert_suitable_main_function(
     value: &ValueConstructor,
     module_name: &EcoString,
+    origin: Origin,
     target: Target,
 ) -> Result<(), crate::Error> {
-    let not_found = || crate::Error::ModuleDoesNotHaveMainFunction {
-        module: module_name.clone(),
-    };
-
     // The value must be a module function
     let ValueConstructorVariant::ModuleFn {
         arity,
@@ -1566,7 +1661,10 @@ fn assert_suitable_main_function(
         ..
     } = &value.variant
     else {
-        return Err(not_found());
+        return Err(crate::Error::ModuleDoesNotHaveMainFunction {
+            module: module_name.clone(),
+            origin,
+        });
     };
 
     // The target must be supported
@@ -1585,5 +1683,13 @@ fn assert_suitable_main_function(
         });
     }
 
+    // The function must be public, or trying to run it would result in a
+    // runtime crash
+    if !value.publicity.is_importable() {
+        return Err(crate::Error::MainFunctionIsPrivate {
+            module: module_name.clone(),
+        });
+    }
+
     Ok(())
 }
