@@ -3,7 +3,7 @@ use super::{
     expression::{ArgumentKind, CallKind},
 };
 use crate::{
-    ast::{BinOp, Layer, SrcSpan, TodoKind},
+    ast::{BinOp, BitArraySegmentTruncation, Layer, SrcSpan, TodoKind},
     build::Target,
     type_::Type,
 };
@@ -145,10 +145,11 @@ impl ModuleSuggestion {
 
 #[derive(Debug, Eq, PartialEq, Clone)]
 pub enum Error {
-    SrcImportingTest {
+    InvalidImport {
         location: SrcSpan,
-        src_module: crate::error::Name,
-        test_module: crate::error::Name,
+        importing_module: EcoString,
+        imported_module: EcoString,
+        kind: InvalidImportKind,
     },
 
     BitArraySegmentError {
@@ -628,6 +629,14 @@ pub enum Error {
         operator: BinOp,
         location: SrcSpan,
     },
+
+    DoubleVariableAssignmentInBitArray {
+        location: SrcSpan,
+    },
+
+    NonUtf8StringAssignmentInBitArray {
+        location: SrcSpan,
+    },
 }
 
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
@@ -661,6 +670,13 @@ pub enum LiteralCollectionKind {
     Record,
 }
 
+#[derive(Debug, Clone, Copy, PartialEq, Eq)]
+pub enum InvalidImportKind {
+    SrcImportingTest,
+    SrcImportingDev,
+    DevImportingTest,
+}
+
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
 pub enum Named {
     Type,
@@ -678,16 +694,16 @@ pub enum Named {
 impl Named {
     pub fn as_str(self) -> &'static str {
         match self {
-            Named::Type => "type",
-            Named::TypeAlias => "type alias",
-            Named::TypeVariable => "type variable",
-            Named::CustomTypeVariant => "type variant",
-            Named::Variable => "variable",
-            Named::Argument => "argument",
-            Named::Label => "label",
-            Named::Constant => "constant",
-            Named::Function => "function",
-            Named::Discard => "discard",
+            Named::Type => "Type",
+            Named::TypeAlias => "Type alias",
+            Named::TypeVariable => "Type variable",
+            Named::CustomTypeVariant => "Type variant",
+            Named::Variable => "Variable",
+            Named::Argument => "Argument",
+            Named::Label => "Label",
+            Named::Constant => "Constant",
+            Named::Function => "Function",
+            Named::Discard => "Discard",
         }
     }
 }
@@ -739,7 +755,6 @@ pub enum Warning {
     UnusedValue {
         location: SrcSpan,
     },
-
     NoFieldsRecordUpdate {
         location: SrcSpan,
     },
@@ -816,9 +831,9 @@ pub enum Warning {
         layer: Layer,
     },
 
-    UnreachableCaseClause {
+    UnreachableCasePattern {
         location: SrcSpan,
-        reason: UnreachableCaseClauseReason,
+        reason: UnreachablePatternReason,
     },
 
     /// This happens when someone tries to write a case expression where one of
@@ -951,6 +966,23 @@ pub enum Warning {
     JavaScriptIntUnsafe {
         location: SrcSpan,
     },
+
+    /// When we are trying to use bool assert on a literal value. For example:
+    /// ```gleam
+    /// assert True
+    ///        ^ The programmer knows this will never panic, so it's useless
+    /// ```
+    AssertLiteralValue {
+        location: SrcSpan,
+    },
+
+    /// When a segment has a constant value that is bigger than its size and we
+    /// know for certain is going to be truncated.
+    ///
+    BitArraySegmentTruncatedValue {
+        truncation: BitArraySegmentTruncation,
+        location: SrcSpan,
+    },
 }
 
 #[derive(Debug, Eq, Copy, PartialEq, Clone, serde::Serialize, serde::Deserialize)]
@@ -968,6 +1000,7 @@ pub enum FeatureKind {
     LetAssertWithMessage,
     VariantWithDeprecatedAnnotation,
     JavaScriptUnalignedBitArray,
+    BoolAssert,
 }
 
 impl FeatureKind {
@@ -996,6 +1029,8 @@ impl FeatureKind {
 
             FeatureKind::JavaScriptUnalignedBitArray => Version::new(1, 9, 0),
             FeatureKind::UnannotatedFloatSegment => Version::new(1, 10, 0),
+
+            FeatureKind::BoolAssert => Version::new(1, 11, 0),
         }
     }
 }
@@ -1023,7 +1058,7 @@ pub enum TodoOrPanic {
 }
 
 #[derive(Debug, Clone, Copy, PartialEq, Eq, serde::Serialize, serde::Deserialize)]
-pub enum UnreachableCaseClauseReason {
+pub enum UnreachablePatternReason {
     /// The clause is unreachable because a previous pattern
     /// matches the same case.
     DuplicatePattern,
@@ -1037,7 +1072,7 @@ impl Error {
     // Location where the error started
     pub fn start_location(&self) -> u32 {
         match self {
-            Error::SrcImportingTest { location, .. }
+            Error::InvalidImport { location, .. }
             | Error::BitArraySegmentError { location, .. }
             | Error::UnknownVariable { location, .. }
             | Error::UnknownType { location, .. }
@@ -1104,7 +1139,9 @@ impl Error {
             | Error::ErlangFloatUnsafe { location }
             | Error::FloatOperatorOnInts { location, .. }
             | Error::IntOperatorOnFloats { location, .. }
-            | Error::StringConcatenationWithAddInt { location } => location.start,
+            | Error::StringConcatenationWithAddInt { location }
+            | Error::DoubleVariableAssignmentInBitArray { location }
+            | Error::NonUtf8StringAssignmentInBitArray { location } => location.start,
 
             Error::UnknownLabels { unknown, .. } => {
                 unknown.iter().map(|(_, s)| s.start).min().unwrap_or(0)
@@ -1136,7 +1173,7 @@ impl Warning {
         }
     }
 
-    fn location(&self) -> SrcSpan {
+    pub(crate) fn location(&self) -> SrcSpan {
         match self {
             Warning::Todo { location, .. }
             | Warning::ImplicitlyDiscardedResult { location, .. }
@@ -1157,7 +1194,7 @@ impl Warning {
             | Warning::InefficientEmptyListCheck { location, .. }
             | Warning::TransitiveDependencyImported { location, .. }
             | Warning::DeprecatedItem { location, .. }
-            | Warning::UnreachableCaseClause { location, .. }
+            | Warning::UnreachableCasePattern { location, .. }
             | Warning::CaseMatchOnLiteralCollection { location, .. }
             | Warning::CaseMatchOnLiteralValue { location, .. }
             | Warning::OpaqueExternalType { location, .. }
@@ -1168,7 +1205,9 @@ impl Warning {
             | Warning::UnreachableCodeAfterPanic { location, .. }
             | Warning::RedundantPipeFunctionCapture { location, .. }
             | Warning::FeatureRequiresHigherGleamVersion { location, .. }
-            | Warning::JavaScriptIntUnsafe { location, .. } => *location,
+            | Warning::JavaScriptIntUnsafe { location, .. }
+            | Warning::AssertLiteralValue { location, .. }
+            | Warning::BitArraySegmentTruncatedValue { location, .. } => *location,
         }
     }
 
@@ -1443,7 +1482,7 @@ fn unify_enclosed_type_test() {
             crate::type_::float(),
             Err(UnifyError::CouldNotUnify {
                 expected: crate::type_::string(),
-                given: crate::type_::bits(),
+                given: crate::type_::bit_array(),
                 situation: Some(UnifyErrorSituation::CaseClauseMismatch {
                     clause_location: SrcSpan::default()
                 })
