@@ -230,7 +230,7 @@ impl Names {
         module: &'a EcoString,
         name: &'a EcoString,
     ) -> NameContextInformation<'a> {
-        let key: (EcoString, EcoString) = (module.clone(), name.clone());
+        let key = (module.clone(), name.clone());
 
         // There is a local name for this value, use that.
         if let Some(name) = self.local_value_constructors.get_by_left(&key) {
@@ -248,6 +248,10 @@ impl Names {
     pub fn is_imported(&self, module: &str) -> bool {
         self.imported_modules.contains_key(module)
     }
+
+    pub fn get_type_variable(&self, id: u64) -> Option<&EcoString> {
+        self.type_variables.get(&id)
+    }
 }
 
 #[derive(Debug)]
